USE Master;
DROP DATABASE TestRoles;

CREATE DATABASE TestRoles
ON
  (NAME = tr_data,
   FILENAME = 'C:\Temp\TestRoles.mdf',
   SIZE = 2MB,
   MAXSIZE = 10MB,
   FILEGROWTH = 2MB
)
LOG ON
  (NAME = tr_log,
   FILENAME = 'C:\Temp\TestRoles.ldf',
   SIZE = 1MB,
   MAXSIZE = 5MB,
   FILEGROWTH = 1MB
);
USE TestRoles;

# Customers.
CREATE TABLE Customers (
    CompanyName     VARCHAR(60)   PRIMARY KEY,
    FEIN            CHAR(9)       NOT NULL,
    AccountBalance  MONEY         NOT NULL
);

# Customers.
INSERT INTO Customers VALUES ('Mike''s Software Emporium', '123456789', 100.00);
INSERT INTO Customers VALUES ('Everlasting Software Inc',  '282983755',  50.00);
INSERT INTO Customers VALUES ('Perplexment Enterprises',   '499237742',  10.00);
INSERT INTO Customers VALUES ('Abend Ender',               '765487830', -45.00);

# Make some users and roles.
#sp_addlogin 'merlin';
sp_addlogin 'a_clerk';
sp_addlogin 'an_oeclerk';
sp_grantdbaccess 'merlin';
sp_grantdbaccess 'a_clerk';
sp_grantdbaccess 'an_oeclerk';
sp_helpuser;

sp_addrole 'Clerks';
sp_addrole 'OrderEntryClerks';
sp_helprole;

# Add OrderEntryClerks and a_clerk to Clerks.
sp_addrolemember 'Clerks', 'OrderEntryClerks';
sp_addrolemember 'Clerks', 'a_clerk';

# Add an_oeclerk to OrderEntryClerks.
sp_addrolemember 'OrderEntryClerks', 'an_oeclerk';

# Allow Clerks to look at Customers.
GRANT SELECT ON Customers TO Clerks;

# Prevent OrderEntryClerks from looking at Customers.AccountBalance.
DENY SELECT ON Customers (AccountBalance) TO OrderEntryClerks;

# If you log on as a_clerk, this should work:
#	USE TestRoles;
#	SELECT * FROM Customers;
#
# If you log on as an_oeclerk, this should work:
#	USE TestRoles;
#	SELECT CompanyName, FEIN FROM Customers;
# This should not work:
#	SELECT * FROM Customers;
